require 'json'
require 'cucumber'
require 'tempfile'
require 'net/http'

module API
	class API

		def initialize()
			@data = {}
			@active = false
			@username = ""
			@userpass = ""
			@headers = {}
		end

		def deinitialize()

			if (@data['management_pack'])
				@data['management_pack'].each_key {|key|
					self.delete("management_pack", key)
				}
				@data.delete("management_pack")
			end

			@data.each {|table, objects|
				objects.each_key {|key|
					self.delete(table, key)
				}
			}

		end

		def set_user(name, pass)
			@username = name
			@userpass = pass
		end

		def add_header(key, value)
			@headers[key] = value
		end

		def request(uri, type = 'post', data = nil)

			Net::HTTP.start(uri.host, uri.port, :use_ssl => true, :verify_mode => OpenSSL::SSL::VERIFY_NONE) do |http|

				request = nil
				if type == 'post'
					request = Net::HTTP::Post.new(uri)
				elsif type == 'delete'
					request = Net::HTTP::Delete.new(uri)
				end

				request.basic_auth @username, @userpass
				request.content_type = 'application/json'

				@headers.each {|key, value|
					request.add_field(key, value)
				}

				if (!(data.nil?))
					request.body = data.to_json
				end

				response = http.request(request)
				if (type == 'delete' && response.code.to_i == 404)
					# Silenced because of the service removal causes a 404 error
					# due to the service already being removed during the removal of the hostgroup
				elsif response.code.to_i >= 300
					puts "Failed HTTP #{type} #{uri} (#{response.code})"
					data = JSON.parse(response.body)
					raise data['full_error']
				end

			end

		end

		def active?()
			return @active
		end

		def create(table, key, data)

			# Keep track of data created so that we may remove after the test
			@active = true
			if (@data[table].nil?)
				@data[table] = {}
			end
			@data[table][key] = data

			uri = URI("https://#{SERVER}/api/config/#{table}")
			self.request(uri, 'post', data)
		end

		def delete(table, key)
			uri = URI("https://#{SERVER}/api/config/#{table}/#{URI.escape(key)}")
			self.request(uri, 'delete')
		end

		# Saves the data via the HTTP API
		def save()
			uri = URI("https://#{SERVER}/api/config/change")
			self.request(uri, 'post')
		end

	end
end
